import click
import os

import simplejson

from rastless.config import Cfg
from rastless.core.colormap import create_colormap


@click.command()
@click.argument('sld_file', type=click.Path(exists=True))
@click.option("-n", "--name", help="Name of the colormap, otherwise take the filename")
@click.option("-d", "--description", help="Add description")
@click.option("-l", "--legend-image", help="Filepath to png legend image")
@click.pass_obj
def add_colormap(cfg: Cfg, sld_file, name, description, legend_image):
    """Add a SLD file"""
    if not name:
        name = os.path.basename(sld_file.split(".")[0])
    try:
        color_map = create_colormap(name, sld_file, description, legend_image)
        cfg.db.add_color_map(color_map)
    except Exception as e:
        click.echo(f"SLD File could not be converted. Reason: {e}")


@click.command()
@click.option("-n", "--name", help="Name of the colormap", required=True)
@click.pass_obj
def delete_colormap(cfg: Cfg, name):
    """Remove a SLD file"""
    cfg.db.delete_color_map(name)


@click.command()
@click.pass_obj
def list_colormaps(cfg: Cfg):
    cms = cfg.db.get_color_maps()
    click.echo(simplejson.dumps(cms, indent=4, sort_keys=True))




from rastless.db.handler import Database
from rastless.core.s3 import S3Bucket
from rastless import settings


class Cfg:
    def __init__(self, table_name, bucket_name):
        self.db = Database(table_name)
        self.s3 = S3Bucket(bucket_name)



import click
import simplejson

from rastless.config import Cfg
from rastless.db.base import str_uuid
from rastless.db.models import LayerModel, LayerStepModel, PermissionModel
from rastless.core.cog import layer_is_valid_cog, create_s3_cog_info, upload_cog_file, transform_upload_cog, \
    get_layer_info


@click.command()
@click.pass_obj
@click.option("-cl", "--client", required=True, type=str, help="Define frontend client, which consumes the layers")
@click.option("-pr", "--product", required=True, type=str, help="Product abbreviation e.g tur, sdb")
@click.option("-t", "--title", required=True, type=str, help="Title which is displayed in the frontend")
@click.option("-id", "--layer-id", default=str_uuid, type=str, help="Predefined uuid otherwise self generated")
@click.option("-cm", "--colormap", type=str, help="SLD colormap name")
@click.option("-u", "--unit", type=str, help="Unit abbreviation e.g. m or FTU")
@click.option("-b", "--background-id", type=str, help="Layer uuid of the background layer")
@click.option("-d", "--description", type=str, help="Description to better identify the layer")
@click.option("-r", "--region-id", default=1, type=int, help="Region ID of api-layer service")
@click.option("-pe", "--permissions", type=str, multiple=True,
              help="Keycloak role permissions in the following form: User -> user#<unique username>"
                   " e.g. user#siegmann@eomap.de, Role -> role#<keycloak client>:<client role>"
                   " e.g. role#hypos:full-access")
def create_layer(cfg: Cfg, permissions, **kwargs):
    """Create layer. This has to be done before adding timesteps"""
    layer = LayerModel.parse_obj(kwargs)
    cfg.db.add_layer(layer)

    permission_models = [PermissionModel(permission=permission, layer_id=layer.layer_id) for permission in permissions]
    cfg.db.add_permissions(permission_models)

    click.echo(f"Created layer with id: {layer.layer_id}")
    return layer.layer_id


@click.command()
@click.pass_obj
@click.argument('filename', type=click.Path(exists=True))
@click.option("-d", "--datetime", required=True, type=str, help="Admission date")
@click.option("-s", "--sensor", required=True, type=str, help="Sensor e.g. SENT2")
@click.option("-l", "--layer-id", required=True, type=str, help="Created layer uuid")
@click.option("-t", "--temporal-resolution", default="daily", type=str, help="Temporal resolution e.g. daily, monthly")
@click.option("-p", "--profile", type=click.Choice(["deflate", "webp"]), default="deflate",
              help="Compression of the GeoTiff")
def create_timestep(cfg: Cfg, filename, datetime, sensor, layer_id, temporal_resolution, profile):
    """Create timestep entry and upload layer to S3 bucket"""
    s3_cog = create_s3_cog_info(cfg.s3.bucket_name, layer_id, filename)
    layer_info = get_layer_info(filename)

    if layer_is_valid_cog(filename):
        uploaded = upload_cog_file(s3_cog)
    else:
        uploaded = transform_upload_cog(s3_cog, profile)

    if not uploaded:
        raise click.ClickException(f"File {filename} could not be uploaded. Please try again.")

    bbox = [round(x, 6) for x in layer_info.bbox_wgs84]
    mapbox_bbox = [bbox[1], bbox[0], bbox[3], bbox[2]]

    layer_step = LayerStepModel(layer_id=layer_id, cog_filepath=s3_cog.s3_path, datetime=datetime, sensor=sensor,
                                temporal_resolution=temporal_resolution, maxzoom=layer_info.maxzoom,
                                minzoom=layer_info.minzoom,
                                bbox=mapbox_bbox, resolution=layer_info.resolution)
    cfg.db.add_layer_step(layer_step)


@click.command()
@click.option("-cl", "--client", type=str, help="Filter by client")
@click.pass_obj
def list_layers(cfg: Cfg, client):
    """List all layers"""
    layers = cfg.db.list_layers()
    if client:
        filtered_layers = [x for x in layers if x["client"] == client]
    else:
        filtered_layers = layers

    click.echo(simplejson.dumps(filtered_layers, indent=4, sort_keys=True))


@click.command()
@click.pass_obj
@click.option("-l", "--layer-id", required=True, type=str, help="Layer uuid")
def delete_layer(cfg: Cfg, layer_id):
    """Delete a layer with all timestep entries and permissions"""
    click.confirm(f'Do you really want to delete layer {layer_id}?', abort=True)

    layer_steps = cfg.db.get_layer_steps(layer_id)
    cfg.s3.delete_layer_steps(layer_steps)
    cfg.db.delete_layer(layer_id=layer_id)


@click.command()
@click.pass_obj
@click.option("-l", "--layer-id", required=True, type=str, help="Layer uuid")
def layer_exists(cfg: Cfg, layer_id):
    """Ask database if layer_id exists: returns a boolean"""
    response = cfg.db.get_layer("layer", f"layer#{layer_id}")
    click.echo(bool(response))



from pydantic import Field
from typing import List
from decimal import Decimal

from .base import DynamoBaseModel, str_uuid


class LayerModel(DynamoBaseModel):
    layer_id: str = Field(default_factory=str_uuid)
    client: str
    product: str
    title: str
    region_id: int = 1
    unit: str = None
    background_id: str = None
    colormap: str = None
    description: str = None

    _pk_tag = "layer"
    _sk_tag = "layer"
    _sk_value = "layer_id"


class PermissionModel(DynamoBaseModel):
    permission: str
    layer_id: str

    _pk_tag = "permission"
    _pk_value = "permission"
    _sk_tag = "layer"
    _sk_value = "layer_id"


class LayerStepModel(DynamoBaseModel):
    layer_id: str
    cog_filepath: str
    datetime: str
    sensor: str
    resolution: Decimal
    temporal_resolution: str
    maxzoom: int
    minzoom: int
    bbox: List[Decimal]

    _pk_tag = "step"
    _pk_value = "datetime"
    _sk_tag = "layer"
    _sk_value = "layer_id"

    def s3_object_name(self, bucket_name):
        return self.cog_filepath.replace(f"s3://{bucket_name}/", "")


class ColorMap(DynamoBaseModel):
    name: str
    description: str = None
    values: List[Decimal]
    colors: List[List[Decimal]]
    nodata: List[Decimal]
    legend_image: str = None

    _pk_tag = "cm"
    _sk_tag = "cm"
    _sk_value = "name"



import boto3
import os
from typing import List

from rastless.db.models import LayerStepModel


class S3Bucket:
    def __init__(self, bucket_name, region="eu-central-1"):
        self.bucket_name = bucket_name
        self.s3 = boto3.resource('s3', region_name=region)
        self.s3_client = boto3.client('s3')
        self.bucket = self.s3.Bucket(bucket_name)

    def delete_object(self, object_name):
        self.s3.Object(self.bucket_name, object_name).delete()

    def list_bucket_entries(self, prefix=None):
        bucket = self.s3.Bucket(self.bucket_name)
        if prefix:
            files = bucket.objects.filter(Prefix=prefix)
        else:
            files = bucket.objects.all()

        return list(files)

    def delete_layer_steps(self, layer_steps: List[LayerStepModel]):
        for step in layer_steps:
            self.delete_object(step.s3_object_name(self.bucket_name))





import click

from rastless.config import Cfg
from rastless.cli import management, layers, colormaps, permissions
from rastless import settings


@click.group()
@click.option('--dev', is_flag=True)
@click.pass_context
def cli(ctx, dev):
    table_name = settings.RASTLESS_TABLE_NAME_DEV if dev else settings.RASTLESS_TABLE_NAME
    bucket_name = settings.RASTLESS_BUCKET_NAME_DEV if dev else settings.RASTLESS_BUCKET_NAME

    ctx.obj = Cfg(table_name=table_name, bucket_name=bucket_name)


# Management
cli.add_command(management.check_aws_connection)

# Layers
cli.add_command(layers.create_layer)
cli.add_command(layers.create_timestep)
cli.add_command(layers.list_layers)
cli.add_command(layers.delete_layer)
cli.add_command(layers.layer_exists)

# Colormaps
cli.add_command(colormaps.add_colormap)
cli.add_command(colormaps.delete_colormap)
cli.add_command(colormaps.list_colormaps)

# Permissions
cli.add_command(permissions.add_permission)
cli.add_command(permissions.delete_permission)
cli.add_command(permissions.get_permissions)

if __name__ == '__main__':
    cli()



from pydantic import BaseModel, root_validator
import uuid


def camel_case(string: str) -> str:
    return ''.join(word.capitalize() if i > 0 else word for i, word in enumerate(string.split('_')))


def str_uuid():
    return str(uuid.uuid4())


class DynamoBaseModel(BaseModel):
    _pk_tag: str
    _pk_value: str = None
    _sk_tag: str
    _sk_value: str = None
    pk: str = None
    sk: str = None

    @root_validator
    def set_keys(cls, values):
        values["pk"] = values["pk"] or cls.create_tag(cls._pk_tag, cls._pk_value, values)
        values["sk"] = values["sk"] or cls.create_tag(cls._sk_tag, cls._sk_value, values)
        return values

    @classmethod
    def create_tag(cls, tag, value, values):
        return tag if not value else f"{tag}#{values[value]}"

    class Config:
        allow_population_by_field_name = True
        alias_generator = camel_case



import boto3


def check_dynamodb_table_connection(table_name: str) -> [bool, str]:
    dynamodb_client = boto3.client('dynamodb')
    error = ""

    try:
        dynamodb_client.describe_table(TableName=table_name)
        has_access = True
    except Exception as e:
        error = e
        has_access = False

    return has_access, error


def check_bucket_connection(bucket_name: str) -> [bool, str]:
    s3 = boto3.resource('s3')
    error = ""

    try:
        s3.meta.client.head_bucket(Bucket=bucket_name)
        has_access = True
    except Exception as e:
        error = e
        has_access = False

    return has_access, error


if __name__ == '__main__':
    check_dynamodb_table_connection("rastless")
    check_bucket_connection("rastless")


import click
from rastless.main import Cfg
from rastless.core import aws_connection


@click.command()
@click.pass_obj
def check_aws_connection(cfg: Cfg):
    """Check if cli can connect to aws"""
    has_bucket_access, bucket_error = aws_connection.check_bucket_connection(cfg.s3.bucket_name)
    has_db_access, db_error = aws_connection.check_dynamodb_table_connection(cfg.db.table_name)

    if all([has_bucket_access, has_db_access]):
        click.echo("You have access to aws resources!")
    else:
        if not has_db_access:
            click.echo(f"DB ERROR: {db_error}")
        if not has_bucket_access:
            click.echo(f"S3 ERROR: {bucket_error}")



import os
from pathlib import Path

ROOT_DIR = Path(os.path.abspath(__file__)).parent.parent

RASTLESS_TABLE_NAME = os.getenv("RASTLESS_TABLE_NAME", "rastless-prod")
RASTLESS_TABLE_NAME_DEV = os.getenv("RASTLESS_TABLE_NAME_DEV", "rastless-dev")

RASTLESS_BUCKET_NAME = os.getenv("RASTLESS_BUCKET_NAME", "rastless-prod")
RASTLESS_BUCKET_NAME_DEV = os.getenv("RASTLESS_BUCKET_NAME_DEV", "rastless-dev")



import click
import simplejson

from rastless.config import Cfg
from rastless.db.models import PermissionModel


@click.command()
@click.option('-p', '--permission', required=True, type=str,
              help='Role e.g role#<client>:<client_role>, user#<username>')
@click.option('-l', '--layer_ids', help='Layer id', required=True, type=str, multiple=True)
@click.pass_obj
def add_permission(cfg: Cfg, permission, layer_ids):
    """Add a role to one or multiple layers."""
    permissions = [PermissionModel(permission=permission, layer_id=layer) for layer in layer_ids]
    cfg.db.add_permissions(permissions)
    click.echo("Role was successfully added to layers")


@click.command()
@click.option('-p', '--permissions', help='Permission name e.g role#<client>:<client_role>, user#<username>',
              required=True, type=str, multiple=True)
@click.pass_obj
def delete_permission(cfg: Cfg, permissions):
    """Delete one or multiple permissions."""

    for permission in permissions:
        cfg.db.delete_permission(permission)

    click.echo("Roles were successfully deleted")


@click.command()
@click.option('-l', '--layer_id', help='Layer id', type=str)
@click.option('-p', '--permission', type=str,
              help='Role e.g role#<client>:<client_role>, user#<username>')
@click.pass_obj
def get_permissions(cfg: Cfg, layer_id, permission):
    items = []

    if permission:
        items = cfg.db.get_layers_for_permission(permission)

    if layer_id:
        items = cfg.db.get_permission_for_layer_id(layer_id)

    click.echo(simplejson.dumps(items, indent=4, sort_keys=True))



from xml.dom import minidom
import numpy as np
from typing import List
import base64

from rastless.db.models import ColorMap


class Sld:
    def __init__(self, filename):
        self.xml_doc = minidom.parse(filename)
        self.items = self.xml_doc.getElementsByTagName('sld:ColorMapEntry')

    @staticmethod
    def _hex_to_rgb(hex_color) -> tuple:
        hex_value = hex_color.lstrip('#')
        return tuple(int(hex_value[i:i + 2], 16) for i in (0, 2, 4))

    @property
    def hex_colors(self) -> np.array:
        return np.array([entry.attributes["color"].value for entry in self.items if
                         float(entry.attributes["opacity"].value) > 0])

    @property
    def rgb_colors(self) -> np.array:
        return np.array([self._hex_to_rgb(hex_color) for hex_color in self.hex_colors])

    @property
    def values(self) -> np.array:
        return np.array([float(entry.attributes["quantity"].value) for entry in self.items if
                         float(entry.attributes["opacity"].value) > 0])

    @property
    def no_data(self) -> List:
        return [float(entry.attributes["quantity"].value) for entry in self.items if
                float(entry.attributes["opacity"].value) == 0]


def legend_png_to_base64(legend_filepath: str) -> bytes:
    with open(legend_filepath, "rb") as image_file:
        encoded_image = base64.b64encode(image_file.read())
    return encoded_image


def create_colormap(name: str, sld_filepath: str, description: str = None, legend_filepath: str = None) -> ColorMap:
    legend_base64 = None
    if legend_filepath:
        legend_base64 = legend_png_to_base64(legend_filepath)

    sld = Sld(sld_filepath)
    return ColorMap(name=name, values=sld.values.tolist(), colors=sld.rgb_colors.tolist(),
                    nodata=sld.no_data, description=description, legend_image=legend_base64)



import boto3
from boto3.dynamodb.conditions import Key
from rastless.db.models import ColorMap, LayerStepModel, LayerModel, PermissionModel
from pydantic import parse_obj_as
from typing import List, Union


class Database:
    def __init__(self, table_name, resource=None):
        if not resource:
            resource = boto3.resource('dynamodb')

        self.table_name = table_name
        self.resource = resource
        self.table = self.resource.Table(table_name)

    def scan_table(self, pk_startswith, sk_startswith=None):
        filter_expression = Key("pk").begins_with(pk_startswith)
        if sk_startswith:
            filter_expression &= Key("sk").begins_with(sk_startswith)

        scan_kwargs = {
            "FilterExpression": filter_expression
        }

        items_list = []

        done = False
        start_key = None

        while not done:
            if start_key:
                scan_kwargs['ExclusiveStartKey'] = start_key
            response = self.table.scan(**scan_kwargs)
            if items := response.get("Items"):
                items_list.extend(items)
            start_key = response.get('LastEvaluatedKey', None)
            done = start_key is None

        return items_list

    def get_item(self, pk, sk, gsi=None):
        response = self.table.get_item(Key={
            "pk": pk,
            "sk": sk
        })
        return response.get("Item")

    def list_layers(self):
        response = self.table.query(
            KeyConditionExpression=Key('pk').eq("layer")
        )
        return response.get("Items", [])

    def add_permissions(self, permissions: List[PermissionModel]):
        with self.table.batch_writer() as writer:
            for permission in permissions:
                writer.put_item(
                    Item=permission.dict(by_alias=True)
                )

    def get_permission(self, permission, layer_id):
        return self.get_item(f"permission#{permission}", f"layer#{layer_id}")

    def get_layers_for_permission(self, permission):
        response = self.table.query(
            KeyConditionExpression=Key('pk').eq(f"permission#{permission}") & Key('sk').begins_with("layer")
        )
        return response.get("Items", [])

    def get_permission_for_layer_id(self, layer_id):
        response = self.table.query(
            IndexName="gsi1",
            KeyConditionExpression=Key('sk').eq(f"layer#{layer_id}") & Key('pk').begins_with("permission")
        )

        return response.get("Items", [])

    def add_layer(self, layer: LayerModel):
        self.table.put_item(
            Item=layer.dict(by_alias=True, exclude_none=True)
        )

    def get_layer(self, pk, sk) -> Union[LayerModel, None]:
        if item := self.get_item(pk, sk):
            return LayerModel.parse_obj(item)

        return None

    def get_layers(self, layer_ids: set):
        response = self.resource.batch_get_item(
            RequestItems={
                self.table_name: {
                    "Keys": [{"pk": "layer", "sk": f"layer#{layer_id}"} for layer_id in layer_ids]
                }
            }
        )

        return response["Responses"][self.table_name]

    def delete_layer(self, layer_id: str):
        response = self.table.query(
            IndexName="gsi1",
            KeyConditionExpression=Key('sk').eq(f"layer#{layer_id}")
        )
        items = response.get("Items", [])

        with self.table.batch_writer() as writer:
            for item in items:
                writer.delete_item(
                    Key={"sk": item["sk"], "pk": item["pk"]}
                )

    def delete_permission(self, permission: str):
        response = self.table.query(
            KeyConditionExpression=Key('pk').eq(f"permission#{permission}")
        )
        items = response.get("Items", [])

        with self.table.batch_writer() as writer:
            for item in items:
                writer.delete_item(
                    Key={"pk": item["pk"], "sk": item["sk"]}
                )

    def delete_layer_from_layer_permission(self, layer_name: str, permission: str):
        self.table.delete_item(
            Item={"pk": f"permission#{permission}", "sk": f"layer#{layer_name}"}
        )

    def get_layer_step(self, step: str, layer_id: str) -> LayerStepModel:
        response = self.table.get_item(Key={"pk": f"step#{step}", "sk": f"layer#{layer_id}"})
        if item := response.get("Item"):
            return LayerStepModel.parse_obj(item)

    def get_layer_steps(self, layer_id: str) -> List[LayerStepModel]:
        response = self.table.query(
            IndexName="gsi1",
            KeyConditionExpression=Key("sk").eq(f"layer#{layer_id}") & Key("pk").begins_with("step")
        )
        return parse_obj_as(List[LayerStepModel], response.get("Items", []))

    def add_layer_step(self, layer_step: LayerStepModel):
        self.table.put_item(Item=layer_step.dict(by_alias=True))

    def add_color_map(self, color_map: ColorMap):
        self.table.put_item(
            Item=color_map.dict(by_alias=True)
        )

    def get_color_map(self, name: str) -> ColorMap:
        if item := self.get_item(pk="cm", sk=f"cm#{name}"):
            return ColorMap.parse_obj(item)

    def get_color_maps(self):
        response = self.table.query(
            KeyConditionExpression=Key("pk").eq("cm"),
            ProjectionExpression="#name, description",
            ExpressionAttributeNames={'#name': 'name'}
        )
        return response.get("Items", [])

    def delete_color_map(self, name: str):
        self.table.delete_item(
            Key={"pk": f"cm", "sk": f"cm#{name}"}
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rastless', 'rastless.cli', 'rastless.core', 'rastless.db']

package_data = \
{'': ['*']}

install_requires = \
['boto3>=1.20.26,<2.0.0',
 'click>=8.0.3,<9.0.0',
 'moto>=2.3.0,<3.0.0',
 'pydantic>=1.9.0,<2.0.0',
 'rasterio>=1.2.10,<2.0.0',
 'rio-cogeo>=3.0.2,<4.0.0',
 'simplejson>=3.17.6,<4.0.0']

entry_points = \
{'console_scripts': ['rastless = rastless.main:cli']}

setup_kwargs = {
    'name': 'rastless-cli',
    'version': '0.2.2',
    'description': 'A cli for managing data and user access for the cloud application rastless',
    'long_description': 'Rastless-CLI\n=================\n\n##### A cli for managing data and user access for the cloud application rastless\n\n## Table of Content\n\n- [Installation](#installation)\n- [Running the CLI](#running-the-cli)\n- [Commands Overview](#commands-overview)\n- [Accomplishing a running system](#accomplishing-a-running-system)\n\n## Installation\n\nRequires: Python >=3.8, <4.0\n\n```bash\n$ pip install rastless-cli\n```\n\nRastLess has to be configured before you can check if everything works. Make sure that your aws account is configured\nand has access to DynamoDb and S3.\n\nYou can check if everything works correctly by running:\n\n```bash\n$ rastless check-aws-connection\n```\n\nIf it is not working, make sure to configure the aws connection by configuring the aws cli. You need an Access ID and a\nSecret ID from aws to configure. Please check\nthe [official instructions](https://docs.aws.amazon.com/cli/latest/userguide/cli-configure-quickstart.html) for further\nhelp.\n\n##### Developer Installation\n```bash\n$ pip install poetry\n$ git clone https://github.com/EomapCompany/rastless-cli\n$ cd rastless-cli\n$ poetry install\n```\n\nRun tests\n```bash\ncd tests\npoetry run pytest\n```\n\n## Running the CLI\n\nAfter installation you can run the ClI by using:\n```bash\n$ rastless --help\n```\n\nYou can decide if you want to upload data to the production or development environment.\nBy using the "dev" flag you upload it to development, without to production\n\n```bash\n# Example development\n$ rastless --dev list-layers\n\n# Example production\n$ rastless list-layers\n```\n\n## Commands Overview\n\n| Commands             |                                                     |\n|----------------------|-----------------------------------------------------|\n| add-colormap         | Add a SLD file                                      |\n| add-permission       | Add a role to one or multiple layers                |\n| check-aws-connection | Check if cli can connect to aws                     |\n| create-layer         | Create layer                                        |\n| create-timestep      | Create timestep entry and upload layer to S3 bucket |\n| delete-colormap      | Remove a SLD file                                   |\n| delete-layer         | Delete a layer with all timestep entries            |\n| delete-permission    | Delete one or multiple permissions                  |\n| list-layers          | List all layers                                     |\n\n## Accomplishing a running system\n\n#### 1. Check if you have access to the system\n```bash\n$ rastless check-aws-connection\n```\n\n#### 2. Create a new layer\n- All inputs are strings. You have to take care, that the element exists in the database e.g. the colormap name.\n- Multiple permissions can be set by using multiple -pe flags\n```bash\n$ rastless create-layer -cl hypos -pr tur -t Turbidity -cm log75_C2S8_32bit -u FTU -b <rgb uuid> -d "Some description" -r 1 -pe user#marcel -pe role#hypos:full-access\n```\nIt will return a new uuid which you need to store, in order to upload timesteps to the particular layer\n\n#### 3. Upload Timesteps for layer\n```bash\n$ rastless create-timestep -d 2020-01-01T15:00:00 -s SENT2 -l <layer uuid> -t daily -p deflate\n```',
    'author': 'Marcel Siegmann',
    'author_email': 'siegmann@eomap.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from rio_cogeo.cogeo import cog_validate, cog_info, cog_translate
import boto3
from dataclasses import dataclass
from rasterio.io import MemoryFile
from rio_cogeo.profiles import cog_profiles
from typing import List
from pyproj import Transformer
import os


@dataclass
class S3Cog:
    filepath: str
    filename: str
    bucket_name: str
    s3_object_name: str
    s3_path: str


@dataclass
class LayerInfo:
    bbox_wgs84: List[float]
    resolution: float
    minzoom: int
    maxzoom: int


def create_s3_cog_info(bucket_name: str, layer_id: str, filepath: str) -> S3Cog:
    filename = os.path.basename(filepath)
    object_name = f"layer/{layer_id}/{filename}"
    s3_path = f"s3://{bucket_name}/{object_name}"
    return S3Cog(s3_object_name=object_name, s3_path=s3_path, bucket_name=bucket_name, filename=filename,
                 filepath=filepath)


def upload_cog_file(s3_cog: S3Cog) -> bool:
    s3_client = boto3.client('s3')

    try:
        s3_client.upload_file(s3_cog.filepath, s3_cog.bucket_name, s3_cog.s3_object_name)
    except Exception:
        return False
    return True


def transform_upload_cog(s3_cog: S3Cog, cog_profile: str) -> bool:
    client = boto3.client("s3")
    dst_profile = cog_profiles.get(cog_profile)

    try:
        with MemoryFile() as mem_dst:
            cog_translate(s3_cog.filepath, mem_dst.name, dst_profile, in_memory=True, web_optimized=True)
            client.upload_fileobj(mem_dst, s3_cog.bucket_name, s3_cog.s3_object_name)
    except Exception:
        return False
    return True


def layer_is_valid_cog(filepath: str) -> bool:
    result = cog_validate(filepath)
    return result == (True, [], [])


def pairwise(iterable):
    a = iter(iterable)
    return zip(a, a)


def transform_bbox(bbox, in_proj: str, out_proj: str = "EPSG:4326") -> List[float]:
    transformer = Transformer.from_crs(in_proj, out_proj)
    bbox_wgs84 = [transformer.transform(x, y) for x, y in pairwise(bbox)]

    return list(sum(bbox_wgs84, ()))


def get_layer_info(filename: str) -> LayerInfo:
    result = cog_info(filename)
    geo_info = result["GEO"]
    bbox_wgs84 = transform_bbox(geo_info["BoundingBox"], geo_info["CRS"])
    bbox_wgs84 = [round(x, 6) for x in bbox_wgs84]

    return LayerInfo(bbox_wgs84=bbox_wgs84, resolution=float(geo_info["Resolution"][0]),
                     maxzoom=geo_info["MaxZoom"], minzoom=geo_info["MinZoom"])
